--[[
	HUD dialog list
	
	Presents a transparent display like main display in HUD
]]

local isShowing = false
function cargolist.ui:CreateHUD ()
	-- Check if this list is already being displayed
	if isShowing then
		ProcessEvent ("CARGOLIST_RELOAD")
		return
	end
	
	-- Build Cargo Display Matrix
	local itemList = {}
	local frame = iup.vbox {
		iup.hbox {
			iup.label {title="Cargo", font=cargolist.ui.fontSmall, fgcolor="255 150 150 100 *", size="200x"},
			iup.label {title="Distance", font=cargolist.ui.fontSmall, fgcolor="255 150 150 100 *", size="50x"};
		};
		size = "300x"
	}

	local function Close ()
		UnregisterEvent (frame, "CARGOLIST_RELOAD")
		UnregisterEvent (frame, "CARGOLIST_SET")
		UnregisterEvent (frame, "CARGOLIST_CLOSE")
		UnregisterEvent (frame, "SECTOR_CHANGED")
		UnregisterEvent (frame, "ENTERED_STATION")
		frame.visible = "NO"
		cargolist.data.scanner = nil
		cargolist.util:SetLocks (false)
		isShowing = false
	end
	
	local top, bottom
	local function SetSelectedMatrixRow (row)
		-- adjust row for relative top
		row = row - top
		local i
		for i=1, #itemList do
			if i == row then
				itemList [i][1].fgcolor = cargolist.ui.fgcolor_selected
				itemList [i][2].fgcolor = cargolist.ui.fgcolor_selected
			else
				itemList [i][1].fgcolor = cargolist.ui.fgcolor_unselected
				itemList [i][2].fgcolor = cargolist.ui.fgcolor_unselected
			end
		end
	end
	
	local function ClearData ()
		local item
		while #itemList > 0 do
			item = table.remove (itemList, 1)
			iup.Detach (item)
		end
		itemList = {}
	end
	
	local function ReloadData (doSort)
		if doSort == nil then doSort = true end
	
		-- Determine view window
		local list = cargolist.data.itemList
		local current = cargolist.data.currentDrop
		top = current - 4
		if top < 0 then top = 0 end
		bottom = top + 11
		if bottom > #list + 1 then bottom = #list + 1 end
		
		-- Set up viewing window
		if doSort then
			cargolist.util:SortDrops (list)
		end
		ClearData ()
		local k, v, item, dist
		for k=1, #list do
			-- If item is within viewing window, add to HUD
			if k>top and k<bottom then
				v = list [k]
				item = iup.hbox {
					iup.label {title=tostring (v.name), font=cargolist.ui.fontSmall, fgcolor=cargolist.ui.fgcolor_unselected, size="200x"},
					iup.label {title=tostring (math.floor (v.dist)) .. "m", font=cargolist.ui.fontSmall, fgcolor=cargolist.ui.fgcolor_unselected, size="50x"};
					scanItem = v
				}
				item.radarId = v.id
				item.radarType = v.type
				item.UpdateDistance = function (self)
					if self.scanItem:Update () then
						self [2].title = tostring (math.floor (self.scanItem.dist)) .. "m"
						return false
					else
						return true
					end
				end
				table.insert (itemList, item)
				iup.Append (frame, itemList [#itemList])
			end
		end
		if #itemList > 0 and current > 0 then
			SetSelectedMatrixRow (current)
		else
			Close ()
		end
		iup.Refresh (frame)
	end
	
	function frame:RemoveData (row)
		local item = table.remove (itemList, row)
		iup.Detach (item)
		iup.Refresh (frame)
	end
	
	local function DoUpdate ()
		if isShowing and #itemList > 0 then
			Timer ():SetTimeout (1000, function ()
				local k, item
				cargolist.util:SetLocks (true)
				local objectType, objectId = radar.GetRadarSelectionID ()
				gkinterface.GKProcessCommand ("RadarNone")
				local removeLost = false
				for _, item in ipairs (itemList) do
					removeLost = removeLost or item:UpdateDistance ()
				end
				radar.SetRadarSelection (objectType, objectId)
				cargolist.util:SetLocks (false)
				if removeLost then
					for k, item in ipairs (cargolist.data.itemList) do
						if item.dist == 0 then
							table.remove (cargolist.data.itemList, k)
							if cargolist.data.currentDrop > 1 and k <= cargolist.data.currentDrop then
								cargolist.data.currentDrop = cargolist.data.currentDrop - 1
							end
						end
					end
					ReloadData (false)
					cargolist.util:SetTarget ()
				end
				DoUpdate ()
			end)
		end
	end
	
	-- Events
	function frame:OnEvent (event, data)
		if event == "CARGOLIST_RELOAD" then
			ReloadData (data)
			
		elseif event == "CARGOLIST_SET" then
			cargolist.data.currentDrop = data
			if #cargolist.data.itemList > 0 and data > 0 then
				if (data>top+1 and data<bottom-1) or #cargolist.data.itemList < 11 then
					SetSelectedMatrixRow (data)
				else
					ReloadData ()
				end
			else
				Close ()
			end
			iup.Refresh (frame)
			
		elseif event == "CARGOLIST_CLOSE" then
			Close ()
			
		elseif event == "SECTOR_CHANGED" then
			DoUpdate ()
			
		elseif event == "ENTERED_STATION" then
			DoUpdate ()			
		end
	end
	
	-- Display dialog at stored coordinates
	local x = gkini.ReadInt (cargolist.config, "showXPos", gkinterface.GetXResolution () / 2 + 100)
	local y = gkini.ReadInt (cargolist.config, "showYPos", gkinterface.GetYResolution () - 300)
	frame.cx = x
	frame.cy = y
	iup.Append (HUD.cboxlayer, frame)
	ReloadData ()
	frame.visible = "YES"
	isShowing = true
	RegisterEvent (frame, "CARGOLIST_RELOAD")
	RegisterEvent (frame, "CARGOLIST_SET")
	RegisterEvent (frame, "CARGOLIST_CLOSE")
	RegisterEvent (frame, "SECTOR_CHANGED")
	RegisterEvent (frame, "ENTERED_STATION")
	DoUpdate ()
	
	return frame
end